/*** uart-spi.c  -*- C -*- */
/** UART to SPI (master) adapter. */

/*** Ivan Shmakov, 2019, 2021, 2022 */

/** To the extent possible under law, the author(s) have dedicated
 ** all copyright and related and neighboring rights to this software
 ** to the public domain worldwide.  This software is distributed
 ** without any warranty.
 **
 ** You should have received a copy of the CC0 Public Domain Dedication
 ** along with this software.  If not, see
 ** <http://creativecommons.org/publicdomain/zero/1.0/>.
 */

/*** History: */

/** 0.? 2022-10-04 19:45Z
 **     (MACHINE_MEGA8_LIKE): New name for MACHINE_LIKE_MEGA8, for
 **     consistency with (AVR) machine.h.
 **     (MACHINE_MEGA64_LIKE): Likewise.
 **     (UARTSPI_PORTI_SHIFT): Fixed: consider the values of the
 **     MACHINE_MEGA8_LIKE and MACHINE_MEGA64_LIKE macros (was:
 **     the state of them being defined, which is currently always.)
 **     (UARTSPI_PORTI_MASK): Likewise.
 **     (UARTSPI_PORTS_MASK): Likewise.
 **
 ** 0.? 2022-10-02 20:00:42Z
 **     (sfn.bFVeuKN3pNhE9W-B9z5KPfZOiyeb50QMiFVDvQ2sx0k.c)
 **     Add the usual boilerplate.  Minor coding style tweaks
 **     all across the code.
 **     (MACHINE_LIKE_MEGA8): New macro, defaulting to 0.
 **     (MACHINE_LIKE_MEGA64): Likewise.
 **     (UARTSPI_PORTI_SHIFT): Explicitly support the likes of ATmega8
 **     (was: assumed) and ATmega64.
 **     (UARTSPI_PORTI_MASK): Likewise.
 **     (UARTSPI_PORTS_MASK): Likewise.
 **     (UARTSPI_PORTI_MASK): Use unsigned long long constants
 **     (were: long.)  Wrap UARTSPI_PORTI_SHIFT into parentheses.
 **     (UARTSPI_PORTS_MASK): Likewise.
 **     (UARTSPI_TIMER_DIVISOR): New macro, defaulting to 1024L.
 **     (main): Pass UARTSPI_TIMER_DIVISOR as the argument to
 **     machine_timer_setup (was: function without arguments.)
 **
 ** 0.? 2021-09-01 18:10:20Z
 **     (sfn.p2qRiwN2JbhYJN2c5CU047Whoz9jSkJ_vFcZF3wKvQE.c)
 */

/*** Code: */
#include <stdint.h>

#include <machine.h>
#include <readyled.h>

#ifndef MACHINE_MEGA8_LIKE
#define MACHINE_MEGA8_LIKE  (0)
#endif
#ifndef MACHINE_MEGA64_LIKE
#define MACHINE_MEGA64_LIKE (0)
#endif

/** NB: /SS must be output lest multimaster mode is activated */
#ifdef UARTSPI_PORTI_SHIFT
/* do nothing */
#elif MACHINE_MEGA8_LIKE
/**     /SS is PB2 on ATmega8 and the like; MOSI, SCK are PB3, PB5 */
#define UARTSPI_PORTI_SHIFT (10)
#define UARTSPI_PORTI_MASK  (013ULL << (UARTSPI_PORTI_SHIFT))
#define UARTSPI_PORTS_MASK    (1ULL << (UARTSPI_PORTI_SHIFT))
#elif MACHINE_MEGA64_LIKE
/**     /SS is PB0 on ATmega64 and the like; MOSI, SCK are PB2, PB1 */
#define UARTSPI_PORTI_SHIFT  (8)
#define UARTSPI_PORTI_MASK  (007ULL << (UARTSPI_PORTI_SHIFT))
#define UARTSPI_PORTS_MASK    (1ULL << (UARTSPI_PORTI_SHIFT))
#else
#error "FIXME: unknown MCU? define UARTSPI_PORTI_ macros explicitly?"
#endif

#ifndef UARTSPI_TIMER_DIVISOR
#define UARTSPI_TIMER_DIVISOR (1024L)
#endif

#if 0
/** Used for testing */
#define UARTSPI_IDLE_DELAY  (42)
#endif

int
main ()
{
#ifdef UARTSPI_IDLE_DELAY
  int_least8_t idle = 0, read_count = 0;
#endif
  int_least8_t spi_sent_p = 0;

  machine_init ();
  machine_timer_setup (UARTSPI_TIMER_DIVISOR);
  machine_spi_master_init (0);
  machine_uart_setup (0);

  machine_port_setup (0,
                      (UARTSPI_PORTI_MASK | READYLED_PORT_MASK));
  /** Raise /SS */
  machine_port_write ((UARTSPI_PORTS_MASK),
                      (UARTSPI_PORTI_MASK | READYLED_PORT_MASK));

  while (1) {
    machine_wait ();

    /** NB: call machine_timer_ticked_p () on every iteration */
    if (machine_timer_ticked_p ()) {
      readyled_tick ();
#ifdef UARTSPI_IDLE_DELAY
      if (idle == 0 && machine_uart_can_write_p (0)) {
        machine_uart_write (0, read_count % 64 + '@');
      }
      if (--idle < 0) {
        idle += UARTSPI_IDLE_DELAY;
      }
#endif
    }

    /** NB: we assume here that our SPI is faster than our UART
     **     (so virtually no buffering is required)
     */

    if (! machine_spi_ready (0)) {
      continue;
    }

    if (spi_sent_p && machine_uart_can_write_p (0)) {
      /** NB: actually, we can check /SS here instead of spi_sent_p */
      /** Raise /SS */
      machine_port_write (UARTSPI_PORTS_MASK, UARTSPI_PORTS_MASK);
      uint_least8_t v
        = machine_spi_read (0);
      spi_sent_p
        = 0;
      machine_uart_write (0, v);
    }

    if (machine_uart_can_read_p (0)) {
#ifdef UARTSPI_IDLE_DELAY
      ++read_count;
#endif
      readyled_blink ();

      /** Lower /SS */
      machine_port_write (0, UARTSPI_PORTS_MASK);
      uint_least8_t u
        = machine_uart_read (0);
      machine_spi_write (0, u);
      spi_sent_p
        = 1;
    }
  }

  /* not reached */
}

/*** uart-spi.c ends here */
