/*** machine.h  -*- C -*- */
/** GNU backend for small machine abstraction library. */

/*** Ivan Shmakov, 2019, 2020, 2022 */

/** To the extent possible under law, the author(s) have dedicated
 ** all copyright and related and neighboring rights to this software
 ** to the public domain worldwide.  This software is distributed
 ** without any warranty.
 **
 ** You should have received a copy of the CC0 Public Domain Dedication
 ** along with this software.  If not, see
 ** <http://creativecommons.org/publicdomain/zero/1.0/>.
 */

/*** History: */

/** 0.? 2022-10-02 21:15Z
 **     Minor coding style tweaks all across the code.
 **     (machine_timer_setup): Require a single divisor argument.
 **     (machine_spi_master_p): Declare function.
 **     (machine_spi_ready_p): New name for machine_spi_ready (old one
 **     retained as a macro.)
 **     (machine_port_map_t): New type, currently defined as
 **     uint_least64_t.
 **     (machine_port_setup): Use machine_port_map_t.
 **     (machine_port_read): Likewise.
 **     (machine_port_write): Likewise.
 **
 ** 0.? 2020-03-14
 **     Prepended channel argument to SPI and UART functions declarations.
 **     (machine_eeprom_read): New function.
 **     (machine_eeprom_write): Likewise.
 */

/*** Code: */
#ifndef MACHINE_H
#define MACHINE_H

#include <stdint.h>
#include <stddef.h>             /* for ptrdiff_t */

void machine_init (void);
void machine_wait (void);

void machine_timer_setup (uint_least16_t);
int_least8_t machine_timer_ticked_p (void);

uint_least8_t machine_eeprom_read (ptrdiff_t);
void machine_eeprom_write (ptrdiff_t, uint_least8_t);

int_least8_t machine_spi_master_p (uint_least8_t);
void machine_spi_master_init (uint_least8_t);
void machine_spi_slave_init (uint_least8_t);
/** NB: compatibility alias */
#define machine_spi_ready machine_spi_ready_p
int_least8_t machine_spi_ready_p (uint_least8_t);
uint_least8_t machine_spi_read (uint_least8_t);
void machine_spi_write (uint_least8_t, uint_least8_t);

void machine_uart_setup (uint_least8_t);
int_least8_t machine_uart_can_read_p (uint_least8_t);
int_least8_t machine_uart_can_write_p (uint_least8_t);
int_least16_t machine_uart_read (uint_least8_t);
void machine_uart_write (uint_least8_t, uint_least8_t);

#if 1
typedef uint_least64_t machine_port_map_t;
#else
typedef uint_least32_t machine_port_map_t;
#endif

void machine_port_setup (machine_port_map_t, machine_port_map_t);
machine_port_map_t machine_port_read (uint_least8_t, machine_port_map_t);
void machine_port_write (machine_port_map_t, machine_port_map_t);

#endif
/*** machine.h ends here */
